# Vector Search

This project performs vector search using Python and Google Cloud services like BigQuery. Follow the steps below to set up the environment, install dependencies, and authenticate with Google Cloud.

## Prerequisites

Ensure that the following are installed on your system:

- **Python** (version 3.6 or higher)
- **Pip** (Python's package installer)
- **Google Cloud SDK** for interacting with Google Cloud services

Ensure the following APIs are enabled in your Google Cloud project:
- **BigQuery API**
- **Vertex AI API**

### Install Google Cloud SDK

If you don't have the **Google Cloud SDK** installed, download and install it by following the official instructions [here](https://cloud.google.com/sdk/docs/install).

Once the SDK is installed, authenticate with your Google Cloud account by running the following command in your terminal or command prompt:

```bash
gcloud auth login
```

## Create Table in BigQuery

This document provides the steps to create a table in Google BigQuery with the following specifications:

- **Table Name**: `your_table_name`
- **Columns**:
  - `bpnId`: STRING (TEXT)
  - `embeddings`: FLOAT64 (REPEATED)
  - `text`: STRING (NOT NULL)

## Prerequisites

- Google Cloud account
- Access to BigQuery
- `gcloud` command-line tool installed and configured (optional)

## Steps to Create the Table

1. **Open BigQuery**:
   - Go to the [Google Cloud Console](https://console.cloud.google.com/).
   - Navigate to the BigQuery section.

2. **Select Dataset**:
   - In the left panel, select your project.
   - Click on the dataset where you want to create the table. If you don't have a dataset, create one by clicking on the project name and then "Create Dataset."

3. **Create Table**:
   - Click on the dataset name.
   - Click on "Create Table."

4. **Configure Table Settings**:
   - **Source**: Select "Empty table."
   - **Table Name**: Enter your desired table name (e.g., `your_table_name`).
   - **Schema**: Define the schema as follows:
     - Click on "Add field" to add the first column:
       - **Field name**: `bpnId`
       - **Type**: STRING
       - **Mode**: REQUIRED
     - Click on "Add field" to add the second column:
       - **Field name**: `embeddings`
       - **Type**: FLOAT
       - **Mode**: REPEATED
     - Click on "Add field" to add the third column:
       - **Field name**: `text`
       - **Type**: STRING
       - **Mode**: REQUIRED

5. **Create the Table**:
   - After filling in the details, click on the "Create table" button at the bottom of the page.

## Example SQL Command

You can also create the table using a SQL command in the BigQuery console or through the `bq` command-line tool. Here’s an example:

```sql
CREATE TABLE `your_project_id.your_dataset_id.your_table_name` (
    bpnId STRING NOT NULL,
    embeddings FLOAT64 REPEATED,
    text STRING NOT NULL
);
```

### Run the project

- Create Virtual Environment 
```bash
python3 -m venv venv
```

- Activate Virtual Environment 
```bash
source venv/bin/activate
```

- Install Project Requirements 
```bash
pip install -r requirements.txt
```

- Replace the TableId with Actual TableId in line 91 and 106
```bash
YOUR_TABLE_ID
```

- Uncomment Line 123 to save data in BigQuery Table (Ignore This if already done)
```bash
save_to_bq(generate_embedding())
```

- Run text_embeddings.py If you want to store data in BQ (Ignore This if already done)
```bash
python text_embeddings.py
```

- Once Data is stored run the Flask Server 
```bash
python index.py
```

- Development Server Started now access the URL from Browser and do the testing with Api
```bash
http://127.0.0.1:5000/vector-search?query=Test Query
```

### NOTES

- Make sure to replace the placeholders with exact value.
- Data.docx file is already present in the code.
- If You want to change the data you are free to replace the content of the file.
- We are taking every paragraph and generating its embeddings, You are free to change this logic and define your preferred chunk size.